`timescale 1ns / 1ps

module clk_gen(
   input  rst,
   input  clk_in,
   output clk_out
);


wire clk_in_bufg;
wire dcm0_clkfx;
wire dcm0_clkfx_bufg;
wire [1:0] dcm_locked;

IBUFG ibufg_in (
   .O(clk_in_bufg),
   .I(clk_in)
);

DCM_SP #(
   .CLKDV_DIVIDE(2.0),
   .CLKFX_DIVIDE(25),
   .CLKFX_MULTIPLY(24),
   .CLKIN_DIVIDE_BY_2("FALSE"),
   .CLKIN_PERIOD(62.5),
   .CLKOUT_PHASE_SHIFT("NONE"),
   .CLK_FEEDBACK("NONE"),
   .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
   .DLL_FREQUENCY_MODE("LOW"),
   .DUTY_CYCLE_CORRECTION("TRUE"),
   .PHASE_SHIFT(0),
   .STARTUP_WAIT("FALSE")
) DCM_0 (
   .CLK0(),
   .CLK180(),
   .CLK270(),
   .CLK2X(),
   .CLK2X180(),
   .CLK90(),
   .CLKDV(),
   .CLKFX(dcm0_clkfx),
   .CLKFX180(),
   .LOCKED(dcm_locked[0]),
   .PSDONE(),
   .STATUS(),
   .CLKFB(0),
   .CLKIN(clk_in_bufg),
   .PSCLK(0),
   .PSEN(0),
   .PSINCDEC(0),
   .RST(rst)
);

BUFG bufg_dcm0_clkfx (
   .O(dcm0_clkfx_bufg),
   .I(dcm0_clkfx)
);

wire cntr_rst;
assign cntr_rst = rst | ~dcm_locked[0];
reg [15:0] rst_cnt;
reg rst_dcm;
always @ (posedge dcm0_clkfx_bufg, posedge cntr_rst)
if (cntr_rst)
begin
   rst_cnt <= 0;
   rst_dcm <= 1;
end
else
begin
   if (rst_cnt[15]==0)
      rst_cnt <= rst_cnt+1;
   else if (rst_cnt[15]==1 & dcm_locked[1]==0)
      rst_cnt <= 0;

   if (rst_cnt[15:14]==0)
      rst_dcm <= 1;
   else
      rst_dcm <= 0;
end

wire dcm1_clkfx;
wire dcm1_clkfx_bufg;

DCM_SP #(
   .CLKDV_DIVIDE(2.0),
   .CLKFX_DIVIDE(5),
   .CLKFX_MULTIPLY(8),
   .CLKIN_DIVIDE_BY_2("FALSE"),
   .CLKIN_PERIOD(65.104),
   .CLKOUT_PHASE_SHIFT("NONE"),
   .CLK_FEEDBACK("NONE"),
   .DESKEW_ADJUST("SYSTEM_SYNCHRONOUS"),
   .DLL_FREQUENCY_MODE("LOW"),
   .DUTY_CYCLE_CORRECTION("TRUE"),
   .PHASE_SHIFT(0),
   .STARTUP_WAIT("FALSE")
) DCM_1 (
   .CLK0(),
   .CLK180(),
   .CLK270(),
   .CLK2X(),
   .CLK2X180(),
   .CLK90(),
   .CLKDV(),
   .CLKFX(dcm1_clkfx),
   .CLKFX180(),
   .LOCKED(dcm_locked[1]),
   .PSDONE(),
   .STATUS(),
   .CLKFB(0),
   .CLKIN(dcm0_clkfx_bufg),
   .PSCLK(0),
   .PSEN(0),
   .PSINCDEC(0),
   .RST(rst_dcm)
);

BUFGMUX bufg_dcm1_clkfx (
   .O(dcm1_clkfx_bufg),
   .I0(0),
   .I1(dcm1_clkfx),
   .S(dcm_locked[1])
);

assign clk_out = dcm1_clkfx_bufg;

endmodule

