`timescale 1ns / 1ps

module top_level(
   input  clk_in,
   input  rst_in,
	
	input  [7:0] switch, 
   
   output adc_mclk,
   output adc_bclk,
   output adc_lrclk,
   input  adc_din,

   output dac_mclk,
   output dac_bclk,
   output dac_lrclk,
   output dac_dout
);

wire rst;
assign rst = rst_in;

wire clk;
clk_gen clk_gen(
   .rst(rst_in),
   .clk_in(clk_in),
   .clk_out(clk)
);


reg [7:0] clk_div = 0;
always @ (posedge clk)
   clk_div <= clk_div + 1;
   
assign adc_mclk = clk;
assign dac_mclk = clk;

assign adc_bclk = clk_div[1];
assign dac_bclk = clk_div[1];

assign adc_lrclk = clk_div[7];
assign dac_lrclk = clk_div[7];


wire [4:0] adc_bcnt;
assign adc_bcnt = clk_div[6:2];

wire [4:0] dac_bcnt;
assign dac_bcnt = clk_div[6:2];

wire bclk_rise;
assign bclk_rise = (clk_div[1:0]==2'b01);
wire bclk_fall;
assign bclk_fall = (clk_div[1:0]==2'b11);


reg [23:0] adc_shr;
always @ (posedge clk)
if (bclk_rise==1)
   adc_shr <= {adc_shr[22:0], adc_din};
   
reg adc_valid_l = 0;
reg adc_valid_r = 0;
always @ (posedge clk)
begin
   if (bclk_rise==1 & adc_bcnt==31 & clk_div[7]==1)
      adc_valid_l <= 1;
   else
      adc_valid_l <= 0;

   if (bclk_rise==1 & adc_bcnt==31 & clk_div[7]==0)
      adc_valid_r <= 1;
   else
      adc_valid_r <= 0;
end


wire adc_lfifo_rd;
wire adc_rfifo_rd;




reg [23:0] adc_ldout=0;
reg [23:0] adc_rdout=0;

reg [23:0] dac_ldin=0;
reg [23:0] dac_rdin=0;


//wire [7:0] gateway_in ;
//wire [7:0] gateway_out ;

wire [23:0] gateway_in ;
wire [23:0] gateway_out ;

//assign gateway_in = adc_ldout;
assign gateway_in = adc_ldout;

wire filter_ce; 
assign filter_ce =adc_valid_l;

//Save input samples
always @ (posedge clk)
if (adc_valid_l)
	adc_ldout <= adc_shr;

always @ (posedge clk)
if (adc_valid_r)
	adc_rdout <= adc_shr;


	
//readin results
always @ (posedge clk)
if (adc_lfifo_rd)
	dac_ldin <=gateway_out; //{gateway_out, 16'b0 } ;


always @ (posedge clk)
if (adc_rfifo_rd)
	dac_rdin <= adc_rdout;
	




wire fifo_rd_en;
assign fifo_rd_en = 1;//~adc_lfifo_empty & ~adc_rfifo_empty;

reg [31:0] dac_shr;
always @ (posedge clk)
if (bclk_fall==1)
   if (dac_bcnt==31)
      if (fifo_rd_en==0)
         dac_shr <= 0;
      else if (clk_div[7]==0)
         dac_shr <= {8'b0, dac_ldin};
      else
         dac_shr <= {8'b0, dac_rdin};
   else
      dac_shr <= {dac_shr[30:0], 1'b0};
      
assign adc_lfifo_rd = (fifo_rd_en==1 & bclk_fall==1 & dac_bcnt==31 & clk_div[7]==0);
assign adc_rfifo_rd = (fifo_rd_en==1 & bclk_fall==1 & dac_bcnt==31 & clk_div[7]==1);

assign dac_dout = dac_shr[31];



bandpass_filter instance_name (
    .ce_1(filter_ce), 
    .clk_1(clk), 
    .gateway_in(gateway_in), 
	 .gateway_in1(filter_ce),
    .gateway_out(gateway_out)
    );


/*
scale scale_inst (
    .ce_1(filter_ce), 
    .clk_1(clk), 
    .gateway_in(gateway_in), 
    .gateway_out(gateway_out)
    );
*/

endmodule
